'****************************************************************
'*  Name    : Test3_TC74.BAS                                    *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 29/01/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Reading the Temperature register: TC74_RTR        *
'*          : Using Hardware I2C BusIn                          *
'*          : Set Stanby Mode after reading                     *
'*          : TC74_ReadSeCTemp() command                        *
'****************************************************************

'    Include "D:/MyIncFiles/Amicus18_25K20.inc"
    Include "Amicus18_25K20.inc"
'-------------------------------------------------------------------------------
    Declare SDA_Pin PORTC.4
    Declare SCL_Pin PORTC.3
    Declare Hbus_Bitrate 100
    Declare Slow_Bus On 

'    $define _TempOrgFormat_    ' Temperature original value
'    Include "D:/MyIncFiles/TC74-S.inc"
    Include "TC74-H.inc"    
'-------------------------------------------------------------------------------
    Dim Temperature As Byte
    Dim TemperatureOrg As Byte
    Dim TemperatureW As Word    
    Dim ReturnPolarity As Byte
    Dim ReturnACK As Byte
    Dim SensorAddress As Byte
    Dim ACKNumber As Byte
    Dim SensorOK As Bit    
'===============================================================================
INIT_TC74:
    INTCON = 0
    DelayMS 350     ' This delay is needed to power up the TC74 sensor.

    HRSOut "Test3_TC74.bas file", CR
    HRSOut "TESTING the TC74 Sensor",CR
    HRSOut "Using the TC74_ReadSeCTemp() command with Hardware I2C HBusIn", CR
    HRSOut "Testing the Standby Mode", CR, CR
'===============================================================================
    ACKNumber = 0
    SensorAddress = TC74A4  ' Loading a the first sensor address to have a NACK answer.
    SensorOK = 0
    
MAIN_TC74:
'-------------------------------------------------------------------------------
' Reading the Status & the Temperature register.
' Try to do some resets in the Amicus board while loop is running.
' Power dissipation within the TC74 will cause self-heating and temperature drift error.
' The sensor is changed to Standby Mode after reading the temperature.
' If the sensor is not running with the TC74_ReadSecTemp command then the sensor is broken.

    While 1 = 1
        $ifdef _TempOrgFormat_
        Temperature = TC74_ReadSeCTemp(SensorAddress, ReturnACK)
        $else
        Temperature = TC74_ReadSeCTemp(SensorAddress, ReturnPolarity, ReturnACK)
        $endif

        ' Check the NACK
        If ReturnACK.0 = 1 Then
            If SensorOK = 0 Then
                Inc ACKNumber
                HRSOut "NACK ", Dec ACKNumber, ", there is not any communication with the sensor.", CR
                Inc SensorAddress               ' Try to read another sensor address.
                If SensorAddress > TC74A7 Then
                    SensorAddress = TC74A0
                    HRSOut "There is not valid sensor in the I2C Bus!", CR
                EndIf
            Else
                HRSOut "NACK: There is not any communication with the sensor.", CR
            EndIf
        Else
            ' Check the Ready Mode.
            If ReturnACK.6 = 1 Then
                HRSOut "The sensor is ready",CR
            Else
                HRSOut "The sensor is NOT ready", CR
            EndIf
            TC74_StandbyMode(SensorAddress)        ' Put the sensor in StandBy Mode (NO ReturnACK), after reading.
        EndIf

        ' Print the Polarity and the Temperature result.
        $ifdef _TempOrgFormat_
        HRSOut "Org. Temperature format.", CR
        If Temperature.7 = 1 Then          
            Temperature = ~Temperature
            Inc Temperature    
            HRSOut "Temperature: -", Dec Temperature, " Degrees C", CR
        Else                                    
            HRSOut "Temperature: +", Dec Temperature, " Degrees C", CR
        EndIf
        $else
        If ReturnPolarity = 1 Then          
            HRSOut "Temperature: -", Dec Temperature, " Degrees C", CR
        Else                                    
            HRSOut "Temperature: +", Dec Temperature, " Degrees C", CR
        EndIf        
        $endif
        HRSOut CR
        '-----------------------------------------------------------------------
        DelayMS 2000
    Wend
'===============================================================================
    Stop
   
    
