'****************************************************************
'*  Name    : TC74-S2.inc                                       *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) July 2013 [AFreixanet 2013]         *
'*          : All Rights Reserved                               *
'*  Date    : 17/02/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Driver for the I2C TC74 Temperature Sensor        *
'*          : using the software I2C command of the compiler    *
'*          : See the manual Amicus18 Compiler pages 204 to 208 *
'* Compiler : Proton Basic and Amicus18 compatible              *
'*          : Code especially wrote for PIC18F25K20/PIC18F25K22 *
'*          : with the Amicus18 Board                           *
'****************************************************************
'
' Version 1.1 26/02/2016
' Option to disable the GIE_Save() & GIE_Restore()code.
' Fixed error interrupt with NACK I2C bus.
' New security for Definition of pins of the I2C Bus.
' Version 1.2 11/03/2016
' Modification of PP4 variable because it is used by several programs.
' Modified ANSEL set configuration for SCL and SDA pins because it is used by several programs.
' Fixed error Return_Var for _TempOrgFormat_ option.
'
'-------------------------------------------------------------------------------
'===============================================================================
' This program is free software, you can redistribute it and/or modify.
' The code is offered as-is and the author take any responsibility for its use.
' This program is distributed in the hope that it will be useful, but WITHOUT ANY
' WARRANTY. However, a mention of the author and compiler would be appreciated.
'
' Este programa es libre, puede redistribuirlo y/o modificarlo.
' Se entrega este cdigo tal como esta y el autor NO se hace responsable de
' su utilizacin. Este programa se distribuye con la esperanza de que sea til,
' pero SIN NINGUNA GARANTA. No obstante citar el autor y el compilador
' sera muy apreciado.

' Ce programme est un logiciel libre, vous pouvez le redistribuer et/ou modifier.
' Ce logiciel est offert tel qu'il est, et son auteur NE prend AUCUNE responsabilit
' pour son utilization. Ce programme est distribu dans l'espoir qu'il sera util,
' mais SANS AUCUNE GARANTIE. Cependant mentionner l'auteur et le compileur serait
' aprci.
'===============================================================================
'===============================================================================
' Tiny Serial Digital Thermal Sensor
' (r) 2002 Microchip Technology Inc. DS21462
'
' The TC74 is a serially accessible, digital temperature sensor particularly suited
' for low cost and small form factor applications. Temperature data is converted from
' the onboard thermal sensing element and made available as an 8-bit digital word.
' Communication with the TC74 is accomplished via a 2-wire SMBus/I2C compatible
' serial port. This bus also can be used to implement multi-drop/multi-zone monitoring.
' The SHDN bit in the CONFIG register can be used to activate the low power Standby mode.
' Temperature resolution is 1C. Conversion rate is a nominal 8 samples/sec.
' During normal operation, the quiescent current is 200 uA (typ).
' During standby operation, the quiescent current is 5 uA (typ).
' Small size, low installed cost and ease of use make the TC74 an ideal choice
' for implementing thermal management in a variety of systems.
'
'Features
'    Digital Temperature Sensing in SOT-23-5 or TO-220 Packages
'    Outputs Temperature as an 8-Bit Digital Word
'    Simple SMBus/I2C Serial Port Interface
'    Solid-State Temperature Sensing:
'       - 2C (max.) Accuracy from +25C to +85C
'       - 3C (max.) Accuracy from 0C to +125C
'    Supply Voltage of 2.7V to 5.5V
'    Low Power:
'       - 200 uA (typ.) Operating Current
'       - 5 uA (typ.) Standby Mode Current
'
' Applications
'    Thermal Protection for Hard Disk Drives and other PC Peripherals.
'    PC Card Devices for Notebook Computers
'    Low Cost Thermostat Controls
'    Power Supplies
'    Thermistor Replacement
'
'===============================================================================
' LIBRARY COMMANDS:
' This is a libary using the software I2C commands of the Proton Basic compiler.
' Some new commands are added to make sure the sensor readings.
' All routines are compatible with the PIC18F25K20 and PIC18F25K22 for the Amicus18
' board. For others PICs please check datatsheet.
'===============================================================================
'-------------------------------------------------------------------------------       
'*******************************************************************************
$ifndef _TC74S_INC_
    $define _TC74S_INC_
'-------------------------------------------------------------------------------
    GoTo OverDriverTC74
'*******************************************************************************    
' Uncomment the next line to enable the interrupts protection.
'$define _EnableGIE_Save_

' Uncomment the Next Line To Enable the reading with original format od the sensor only..
'    $define _TempOrgFormat_

'-------------------------------------------------------------------------------
' Definition of variables of the TC74 library.
' Input : 
' Output: 
' Notes :
'
    Dim TC74_I2C_Flags As Byte System
    Dim TC74_ACK As TC74_I2C_Flags.0
    Dim ModeNormalSent As TC74_I2C_Flags.1
    Dim TC74_FirstValidAddress As TC74_I2C_Flags.2
    
    Dim xTC74_Result As Byte System
    Dim xTC74_Address As Byte System
    Dim xTC74_AddressWrite As Byte System 
    Dim xTC74_AddressRead As Byte System 
    Dim xACKReturn As Byte System
    Dim xTCSign As Byte System
    Dim OrgTemperature As Byte System
'-------------------------------------------------------------------------------
' Addresses of diferents standard temperature sensors. 
    $define TC74A0 %01001000
    $define TC74A1 %01001001
    $define TC74A2 %01001010
    $define TC74A3 %01001011
    $define TC74A4 %01001100
    $define TC74A5 %01001101
    $define TC74A6 %01001110
    $define TC74A7 %01001111

' Address + Read value
    $define TC74A0_ADRRead %10010001
    $define TC74A1_ADRRead %10010011
    $define TC74A2_ADRRead %10010101    
    $define TC74A3_ADRRead %10010111
    $define TC74A4_ADRRead %10011001
    $define TC74A5_ADRRead %10011011
    $define TC74A6_ADRRead %10011101
    $define TC74A7_ADRRead %10011111

' Address + Write value
    $define TC74A0_ADRWrite %10010000
    $define TC74A1_ADRWrite %10010010
    $define TC74A2_ADRWrite %10010100    
    $define TC74A3_ADRWrite %10010110
    $define TC74A4_ADRWrite %10011000
    $define TC74A5_ADRWrite %10011010
    $define TC74A6_ADRWrite %10011100
    $define TC74A7_ADRWrite %10011110

' Internal registers
    $define TC74_RTR     $00    ' Address Register: Read Temperature (TEMP REG)
    $define TC74_RWCR    $01    ' Address Register: Read/Write Configuration (CONFIG REG)
    $define TC74_STANDBY $80    ' Setting bit: Standby value 
    $define TC74_NORMAL  $00    ' Setting bit: Normal value     
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
' Definition of pins of the I2C Bus.
' Input Variables   : 
' Output Variables  : 
' Notes             : The pins direction could be changed by the user,
'                   : because this Driver uses all software code.
#ifnSym __SDA_Port
    Declare SDA_Pin PORTC.4           ' SDA connected to this pin
#endIfSym
#ifnSym __SCL_Port
    Declare SCL_Pin PORTC.3           ' SCL connected to this pin
#endIfSym

'-------------------------------------------------------------------------------
'*******************************************************************************
' Declare a variable for INTCON copy.
    $ifndef _xINTCON_Save_Var_
    $define _xINTCON_Save_Var_
    Dim xINTCON_Save As Byte System
    $endif

' Enable this line in the main program befor this library.
'    $define _EnableGIE_Save_

' This new command save and clear the GIE global interrupt.
$ifndef GIE_Save
    $define GIE_Save()          '              
    #ifdef __INTERRUPTS_ENABLED '
    xINTCON_Save = INTCON       '
    INTCONbits_GIE = 0          '
    #endif                      '

$endif

'-------------------------------------------------------------------------------    
' This new command restore the GIE global interrupt.
$ifndef GIE_Restore
    $define GIE_Restore()       '
    #ifdef __INTERRUPTS_ENABLED '
    If xINTCON_Save.7 = 1 Then  '
        INTCONbits_GIE = 1      '
    EndIf                       '
    #endif                      '
    
$endif

'*******************************************************************************
'-------------------------------------------------------------------------------
' Send command Normal Mode. Write Byte Format
' Input Variables   : TC74 Address
' Output Variables  : none
' Syntax            : TC74_WriteNormalMode(TC74 Address)
' Notes             : The command is very slow, it needs 255mS.
'
$define TC74_NormalMode(pValue) _mTC74_WriteNMode pValue

_mTC74_WriteNMode Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mTC74_WriteNMode - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_WriteNMode - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_WriteNMode - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pValue, xTC74_Address                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pValue, xTC74_Address                    
                #endif
                GoSub _mTC74_WriteNMode_Sub
            #endif                
        #endif
    #endif                
Endm

#ifMacro- _mTC74_WriteNMode, _mTC74_SecureReadTemp
_mTC74_WriteNMode_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif  
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressWrite, TC74_RWCR,[TC74_NORMAL]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    DelayMS 255                 ' The sensor needs 250 ms to change to ready mode.
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Send command Standby Mode. Write Byte Format
' Input : address of sensor
' Output: none
' Syntax: TC74_StandbyMode(TC74A4) 
' Notes : 
'
$define TC74_StandbyMode(pValue) _mTC74_WriteStandbyMode pValue

_mTC74_WriteStandbyMode Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mTC74_WriteStandByMode - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_WriteStandByMode - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_WriteStandByMode - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pValue, xTC74_Address                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pValue, xTC74_Address                    
                #endif
                GoSub _mTC74_WriteStandByMode_Sub
            #endif                
        #endif
    #endif                  
Endm

#ifMacro- _mTC74_WriteStandbyMode
_mTC74_WriteStandByMode_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2COut __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressWrite, TC74_RWCR,[TC74_STANDBY]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Read the Status of the sensor
' Input : Sensor Address
' Output: Return Var 
' Syntax: Var = TC74_ReadStatus(TC74A4)
' Notes : 
'
$define TC74_ReadStatus(pValue) _mTC74_ReadStatus pValue

_mTC74_ReadStatus Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mTC74_ReadStatus - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_ReadStatus - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_ReadStatus - (Param1) Should be a Byte or Num8"
                Exitm
            #else
                #if(Prm_1 == Num8)
                    Num_Byte pValue, xTC74_Address                    
                #endif
                #if(Prm_1 == Byte)
                    Byte_Byte pValue, xTC74_Address                    
                #endif
                GoSub _mTC74_ReadStatus_Sub

                #if (_mTC74_ReadStatus_RETURN != 1)
                    #error "_mTC74_ReadStatus - Mandatory return parameter misssing"
                    Exitm
                #else
                    #if(Return_Type != Byte)
                        #error "_mTC74_ReadStatus - Return variable should be a Byte variable"
                        Exitm
                    #else
                        Byte_Byte xACKReturn, Return_Var
                    #endif
                #endif
            #endif                
        #endif
    #endif
Endm

#ifMacro- _mTC74_ReadStatus, _mTC74_Present
_mTC74_ReadStatus_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressRead, TC74_RWCR,[xACKReturn]
    '---------------------------------------------------------------------------    
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
' Read Byte Format. Read the Status before the temperature register.
' Input : Address sensor
' Output: Return Var
' Syntax: Var = TC74_ReadRSTemp(TC74A4 address, Sign)
' Notes : 
'
$ifndef _TempOrgFormat_
$define TC74_ReadSRTemp(pValue, pSign) _mTC74_ReadSRTemp pValue, pSign

_mTC74_ReadSRTemp Macro- pValue, pSign
    #if (Prm_Count > 2)
	   #error "_mTC74_ReadRSTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mTC74_ReadRSTemp - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_ReadRSTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif
            #if (Prm_2 != Byte) && (Prm_2 != Bit)
                #error "_mTC74_ReadRSTemp - (Param2) Should be a Byte or a Bit"
                Exitm
            #endif

            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_ReadSRTemp_Sub
            #if(Prm_2 == Byte)
                Wreg_Byte pSign
            #endif    
            #if (_mTC74_ReadSRTemp_RETURN != 1)
                #error "_mTC74_ReadRSTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_ReadRSTemp - Return variable should be a Byte or word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte xTC74_Result, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word xTC74_Result, Return_Var
                    #endif
                #endif
            #endif
        #endif
    #endif
Endm
    
#ifMacro-  _mTC74_ReadSRTemp, _mTC74_SecureReadTemp
_mTC74_ReadSRTemp_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    xACKReturn = 0
    OrgTemperature = 255        ' False result.
    xTC74_Result = 255          ' False result.
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    ' Reading the Status of the sensor.
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressRead, TC74_RWCR,[xACKReturn]
    '--------------------------------------------------------------------------- 
    If xACKReturn.6 = 0 Then    ' Is the sensor ready?
        $ifdef _EnableGIE_Save_
        GIE_Restore()
        $endif
        WREG = 0
        Return                  ' NO, the sensor is NOT Ready, EXIT the routine.
    EndIf
    DelayUS 5
    '---------------------------------------------------------------------------
    ' Reading the Temperature of the sensor.
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressRead, TC74_RTR,[OrgTemperature]
    '--------------------------------------------------------------------------- 
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    xTC74_Result = OrgTemperature
    If xTC74_Result.7 = 1 Then          
        xTC74_Result = ~xTC74_Result + 1    
        WREG = 1                ' It is a negative Result.
    Else                                    
        WREG = 0                ' It is a positive result.
    EndIf
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
$else
'-------------------------------------------------------------------------------
$define TC74_ReadSRTemp(pValue) _mTC74_ReadSRTemp pValue

_mTC74_ReadSRTemp Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mTC74_ReadRSTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_ReadRSTemp - Too few parameters"
            Exitm
        #else
            #if(Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_ReadRSTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif

            #if (Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if (Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_ReadSRTemp_Sub
            #if (_mTC74_ReadSRTemp_RETURN != 1)
                #error "_mTC74_ReadRSTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_ReadRSTemp - Return variable should be a Byte or word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte OrgTemperature, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word OrgTemperature, Return_Var
                    #endif
                #endif
            #endif
        #endif
    #endif
Endm
    
#ifMacro-  _mTC74_ReadSRTemp, _mTC74_SecureReadTemp
_mTC74_ReadSRTemp_Sub:
    #ifdef WatchDog_Req
    Clrwdt
    #endif 
    xACKReturn = 0
    OrgTemperature = 255        ' False result.
    GoSub SetTC74Addresses_Sub
    '---------------------------------------------------------------------------
    $ifdef _EnableGIE_Save_
    GIE_Save()
    $endif
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressRead, TC74_RWCR,[xACKReturn]
    '--------------------------------------------------------------------------- 
    If xACKReturn.6 = 0 Then        ' Is the sensor ready?
        GoTo _mTC74_ReadSRTempExit  ' NO, the sensor is NOT Ready, EXIT the routine.
    EndIf
    DelayUS 5
    '---------------------------------------------------------------------------
    I2CIn __SDA_Port.__SDA_PIN, __SCL_Port.__SCL_PIN, xTC74_AddressRead, TC74_RTR,[OrgTemperature]
    '--------------------------------------------------------------------------- 
_mTC74_ReadSRTempExit:
    $ifdef _EnableGIE_Save_
    GIE_Restore()
    $endif
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
$endif
'-------------------------------------------------------------------------------
' Secure temperature READ. Using Read Byte Format only.
' The Command read the status, if it is NOT ready Mode, try to change to normal mode once.
' Input : Sensor address
' Output: 2 Return Vars
' Syntax: Temperature = TC74_ReadSeCTemp(Address, Sign, ACKReturn)
' Notes : It is the better way to read the TC74 sensor.
'
$ifndef _TempOrgFormat_
' Calculate the sign and the temperature value.
$define TC74_ReadSeCTemp(pValue, pSign) _mTC74_SecureReadTemp pValue, pSign

_mTC74_SecureReadTemp Macro- pValue, pSign
    #if (Prm_Count > 2)
	   #error "_mTC74_SecureReadTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 2)
            #error "_mTC74_SecureReadTemp - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_SecureReadTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif
            #if (Prm_2 != Byte)
                #error "_mTC74_SecureReadTemp - (Param2) Should be a Byte"
                Exitm
            #endif

            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_SecureReadTemp_Sub

            #if (Prm_2 == Byte)
                Wreg_Byte pSign
            #endif  

            #if (_mTC74_SecureReadTemp_RETURN != 1)
                #error "_mTC74_SecureReadTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_SecureReadTemp - Return variable should be a Byte or word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte xTC74_Result, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word xTC74_Result, Return_Var
                    #endif
                #endif
            #endif    
        #endif
    #endif
Endm

#ifMacro- _mTC74_SecureReadTemp
_mTC74_SecureReadTemp_Sub:
    ModeNormalSent = 0                          ' Initialize a flag
    '---------------------------------------------------------------------------
_mTC74_SecureReadTemp11_Sub:
    GoSub _mTC74_ReadSRTemp_Sub                 ' Read the Status & the Temperature.                     
    '---------------------------------------------------------------------------
    ' Check the fault and try to fix it.
    If xACKReturn.6 = 0 Then                    ' Check if the sensor is NOT ready.
        If ModeNormalSent = 0 Then              ' If is the first time...' The sensor is not ready, try to send a Normal Mode command.
            GoSub _mTC74_WriteNMode_Sub         ' Set the Normal Mode, the delay is included in the routine.
            ModeNormalSent = 1                  ' Don't try again.
            GoTo _mTC74_SecureReadTemp11_Sub    ' Check Status & Read again the Temperature.
        EndIf
        'xACKReturn = 1          ' The sensor is not ready and dosen't accept the Normal Mode.
        xTC74_Result = 255      ' No Temperature value
        WREG = 0                ' +Sign and return.
    EndIf
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
'-------------------------------------------------------------------------------
$else
' Reading the original value of the TC74 sensor.
'-------------------------------------------------------------------------------
$define TC74_ReadSeCTemp(pValue) _mTC74_SecureReadTemp pValue

_mTC74_SecureReadTemp Macro- pValue
    #if (Prm_Count > 1)
	   #error "_mTC74_SecureReadTemp - Too many parameters"
       Exitm
    #else
        #if (Prm_Count < 1)
            #error "_mTC74_SecureReadTemp - Too few parameters"
            Exitm
        #else
            #if (Prm_1 != Num8) && (Prm_1 != Byte)
                #error "_mTC74_SecureReadTemp - (Param1) Should be a Byte or Num8"
                Exitm
            #endif

            #if(Prm_1 == Num8)
                Num_Byte pValue, xTC74_Address                    
            #endif
            #if(Prm_1 == Byte)
                Byte_Byte pValue, xTC74_Address                    
            #endif
            GoSub _mTC74_SecureReadTemp_Sub

            #if (_mTC74_SecureReadTemp_RETURN != 1)
                #error "_mTC74_SecureReadTemp - Mandatory return parameter misssing"
                Exitm
            #else
                #if (Return_Type != Byte) && (Return_Type != Word)
                    #error "_mTC74_SecureReadTemp - Return variable should be a Byte or word variable"
                    Exitm
                #else
                    #if (Return_Type == Byte)
                        Byte_Byte OrgTemperature, Return_Var
                    #endif
                    #if (Return_Type == Word)
                        Byte_Word OrgTemperature, Return_Var
                    #endif
                #endif
            #endif    
        #endif
    #endif
Endm

#ifMacro- _mTC74_SecureReadTemp
_mTC74_SecureReadTemp_Sub:
    ModeNormalSent = 0                          ' Initialize a flag
    '---------------------------------------------------------------------------
_mTC74_SecureReadTemp11_Sub:
    GoSub _mTC74_ReadSRTemp_Sub                 ' Read the Status & the Temperature.                     
    '---------------------------------------------------------------------------
    ' Check the fault and try to fix it.
    If xACKReturn.6 = 0 Then                    ' Check if the sensor is NOT ready.
        If ModeNormalSent = 0 Then              ' If is the first time...' The sensor is not ready, try to send a Normal Mode command.
            GoSub _mTC74_WriteNMode_Sub         ' Set the Normal Mode, the delay is included in the routine.
            ModeNormalSent = 1                  ' Don't try again.
            GoTo _mTC74_SecureReadTemp11_Sub    ' Check Status & Read again the Temperature.
        EndIf
        'xACKReturn = 1          ' The sensor is not ready/NACK and dosen't accept the Normal Mode.
        OrgTemperature = 255    ' No Temperature value, exit the routine.
    EndIf
    '---------------------------------------------------------------------------
    Return
#endIfMacro-
$endif
'-------------------------------------------------------------------------------
' Detect the TC44 sensor checking the status register.
' Input     : None
' Output    : pACKReturn holds 64 value if the TC74 is available (ready)
' Notes     : 
'
$define TC74_Present(pNumberSensor, pACKReturn) '
    xTC74_Address = pNumberSensor               '
    _mTC74_Present                              '
    pACKReturn = xACKReturn

_mTC74_Present Macro-
    GoSub _mTC74_Present_Sub
Endm

#ifMacro- _mTC74_Present
_mTC74_Present_Sub:
    ModeNormalSent = 0
_mTC74_Present22_Sub:
    GoSub _mTC74_ReadStatus_Sub                    
    If xACKReturn.6 = 0 Then
        If ModeNormalSent = 0 Then
            GoSub _mTC74_WriteNMode_Sub
            ModeNormalSent = 1
            GoTo _mTC74_Present22_Sub
        EndIf
        xACKReturn = 1
    Else
        xACKReturn = 0   
    EndIf
    Return
#endIfMacro-
'===============================================================================
' Calculate the Address write and read of the TC74 sensor.
' Input : 
' Output:
' Notes : 
'
SetTC74Addresses_Sub:
    xTC74_AddressWrite = xTC74_Address << 1    
    xTC74_AddressRead = xTC74_AddressWrite | $01
    Return
'===============================================================================
'===============================================================================
' This String is compiled and used only if the 'Include "VersionFile.inc"' is written 
' at the top of your main program. Then this sentence is sent to the Terminal of the Amicus18.
$ifdef _HRSOutVersionFile_inc_
_TC74S_INC_Version: CData "TC74-S2.inc Version 1.2 17/02/2016",0
$endif
'===============================================================================
'-------------------------------------------------------------------------------
'-------------------------------------------------------------------------------
OverDriverTC74:
'===============================================================================
' Set the I2C pins to digital. (Result = 2 lines)
$ifndef _AnselI2CSoftPins_
$define _AnselI2CSoftPins_
#if (__SCL_PORT == PORTA)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTB)
    #if (__SCL_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SCL_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SCL_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SCL_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SCL_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SCL_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SCL_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SCL_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SCL_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SCL_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SCL_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SCL_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SCL_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SCL_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SCL_PORT == PORTE)
    #if (__SCL_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SCL_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SCL_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
'-------------------------------------------------------------------------------
#if (__SDA_PORT == PORTA)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL
        ANSEL.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSEL
        ANSEL.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 5)
        $ifdef _ANSEL
        ANSEL.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTB)
    #if (__SDA_PIN == 0)
        $ifdef _ANSELH
        ANSELH.4 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSELH
        ANSELH.2 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSELH
        ANSELH.0 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.2 = 0
        $endif
    #endif
    #if (__SDA_PIN == 3)
        $ifdef _ANSELH
        ANSELH.1 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.3 = 0
        $endif
    #endif
    #if (__SDA_PIN == 4)
        $ifdef _ANSELH
        ANSELH.3 = 0
        $endif
        $ifdef _ANSELA
        ANSELA.5 = 0
        $endif
    #endif
#endif

#if (__SDA_PORT == PORTC)
    $ifdef _ANSELC
    #if (__SDA_PIN == 2)
        ANSELC.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELC.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELC.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELC.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELC.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELC.7 = 0
    #endif
    $endif
#endif

$ifdef _PORTD
#if (__SDA_PORT == PORTD)
    $ifdef _ANSELD
    #if (__SDA_PIN == 0)
        ANSELD.0 = 0
    #endif
    #if (__SDA_PIN == 1)
        ANSELD.1 = 0
    #endif
    #if (__SDA_PIN == 2)
        ANSELD.2 = 0
    #endif
    #if (__SDA_PIN == 3)
        ANSELD.3 = 0
    #endif
    #if (__SDA_PIN == 4)
        ANSELD.4 = 0
    #endif
    #if (__SDA_PIN == 5)
        ANSELD.5 = 0
    #endif
    #if (__SDA_PIN == 6)
        ANSELD.6 = 0
    #endif
    #if (__SDA_PIN == 7)
        ANSELD.7 = 0
    #endif
    $endif    
#endif
$endif

$ifdef _PORTE
#if (__SDA_PORT == PORTE)
    #if (__SDA_PIN == 0)
        $ifdef _ANSEL       
        ANSEL.5 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.0 = 0
        $endif
    #endif
    #if (__SDA_PIN == 1)
        $ifdef _ANSEL
        ANSEL.6 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.1 = 0
        $endif
    #endif
    #if (__SDA_PIN == 2)
        $ifdef _ANSEL
        ANSEL.7 = 0
        $endif
        $ifdef _ANSELE
        ANSELE.2 = 0
        $endif
    #endif
#endif
$endif
$endif      ' _AnselI2CSoftPins_
'===============================================================================
$endif  ' _TC74S_INC_ 


