'****************************************************************
'*  Name    : Test2_TC74.BAS                                    *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 29/01/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Reading the Temperature register: TC74_RTR        *
'*          : Using Software I2C BusIn                          *
'*          : Verify the NACK & Ready Mode with I2C BusIn       *
'*          : TC74_ReadSeCTemp() command                        *
'****************************************************************

'    Include "D:/MyIncFiles/Amicus18_25K20.inc"
    Include "Amicus18_25K20.inc"
'-------------------------------------------------------------------------------
    Declare SDA_Pin PORTC.4
    Declare SCL_Pin PORTC.3
    Declare Slow_Bus On
    $define _I2CPins_  

'    $define _TempOrgFormat_    ' Temperature original value
'    Include "D:/MyIncFiles/TC74-S.inc"
    Include "TC74-S.inc"    
'-------------------------------------------------------------------------------
    Dim Temperature As Byte
    Dim TemperatureOrg As Byte
    Dim TemperatureW As Word    
    Dim ReturnPolarity As Byte
    Dim ReturnACK As Byte
    Dim SensorAddress As Byte
    Dim ACKNumber As Byte
    Dim SensorOK As Bit    
'===============================================================================
INIT_TC74:
   INTCON = 0
    DelayMS 250     ' This delay is needed to power up the TC74 sensor.

    HRSOut "Test2_TC74.bas file", CR
    HRSOut "TESTING the TC74 Sensor",CR
    HRSOut "Using the TC74_ReadSeCTemp() command with Software I2C BusIn", CR
    HRSOut "Verify the NACK & Ready State", CR, CR
'===============================================================================
    TC74_StandbyMode(TC74A4)        ' Put the sensor in StandBy Mode to have a NOT Ready answer.
    DelayMS 100
    SensorAddress = TC74A0          ' Loading a False sensor address to have a NACK answer.
    ACKNumber = 0
    SensorOK = 0

MAIN_TC74:
'-------------------------------------------------------------------------------
' Reading the Temperature register. Address = TC74_RTR
' Try to do some resets in the Amicus board while loop is running.
'
' 1- When the SensorAddress is incorrect the answer is NACK.
' 2- If the sensor can't be read, the the answer is NOT ready. But the code send 
' automatically the Normal Mode, and the sensor will be ready.
' The Status is always read before reading the temperature register.
'
    While 1 = 1
        $ifdef _TempOrgFormat_
        Temperature = TC74_ReadSeCTemp(SensorAddress, ReturnACK)
        $else
        Temperature = TC74_ReadSeCTemp(SensorAddress, ReturnPolarity, ReturnACK)
        $endif

        ' Check the NACK
        If ReturnACK.0 = 1 Then
            If SensorOK = 0 Then
                Inc ACKNumber                
                HRSOut "NACK ", Dec ACKNumber, ", there is not any communication with the sensor.", CR
                Inc SensorAddress               ' Try to read another sensor address.
                If SensorAddress > TC74A7 Then
                    SensorAddress = TC74A0
                    HRSOut "There is not valid sensor in the I2C Bus!", CR
                EndIf
            Else
                HRSOut "NACK: There is not any communication with the sensor.", CR
            EndIf
        Else
            ' Check the Ready Mode.
            If ReturnACK.6 = 1 Then
                HRSOut "The sensor is ready",CR
            Else
                HRSOut "The sensor is NOT ready", CR
            EndIf
        EndIf

        ' Print the Polarity and the Temperature result.
    $ifndef _TempOrgFormat_
        If ReturnPolarity = 0 Then
            HRSOut "Temperature: +", Dec Temperature
        Else
            HRSOut "Temperature: -", Dec Temperature
        EndIf
        HRSOut " Degrees ", "(", IBin8 Temperature, ")", CR, CR
    $else
        HRSOut "Temperature Org. Format: ", Dec Temperature, CR
        HRSOut " Degrees ", "(", IBin8 Temperature, ")", CR, CR
    $endif
        '-----------------------------------------------------------------------
        DelayMS 3000
    Wend
'===============================================================================
    Stop
   
    
