'****************************************************************
'*  Name    : Test4_TC74.BAS                                    *
'*  Author  : [Alberto Freixanet]                               *
'*  Notice  : Copyright (c) 2016 [AFreixanet]                   *
'*          : All Rights Reserved                               *
'*  Date    : 29/01/2016                                        *
'*  Version : 1.0                                               *
'*  Notes   : Reading the Temperature register: TC74_RTR        *
'*          : Using Software I2C BusIn                          *
'*          : Without ACK control                               *
'*          : TC74_ReadSRTemp() command                         *
'****************************************************************

'    Include "D:/MyIncFiles/Amicus18_25K20.inc"
    Include "Amicus18_25K20.inc"
'-------------------------------------------------------------------------------
    $define _DisableNACK_
'    $define _EnableGIE_Save_
'    $define _TempOrgFormat_

    $ifdef _EnableGIE_Save_
    @#define __INTERRUPTS_ENABLED 1
    $endif

    Declare SDA_Pin PORTC.4
    Declare SCL_Pin PORTC.3
    Declare Slow_Bus On

    $ifdef _DisableNACK_
    Include "TC74-S2.inc"
    $else
    Include "TC74-S.inc"
    $endif    
'-------------------------------------------------------------------------------
    Dim Temperature As Byte
    Dim TemperatureW As Word    
    Dim ReturnPolarity As Byte
    Dim ReturnACK As Byte
    Dim SensorAddress As Byte
    Dim ACKNumber As Byte
    Dim SensorOK As Bit
'===============================================================================
INIT_TC74:
    INTCON = 0
'    ANSEL = 0
'    ANSELH = 0
    DelayMS 200     ' This delay is needed to power up the TC74 sensor.

    HRSOut "Test4_TC74.bas file", CR
    HRSOut "TESTING the TC74 Sensor",CR
    HRSOut "Using the TC74_ReadSRTemp() command with Software I2C", CR
    HRSOut "Using TC74-S.Inc & TC74-S2.Inc", CR, CR
'===============================================================================
    TC74_NormalMode(TC74A4)    ' Put the sensor in Normal Mode to have an answer.
    DelayMS 300
    ' Take care of the TC74 Model of Proteus. Check if there is the sensor address.
    SensorAddress = TC74A4     ' Loading the sensor address (TC74A0 to TC74A7).
    ACKNumber = 0  
    SensorOK = 0

MAIN_TC74:
'-------------------------------------------------------------------------------
' Reading the Temperature register.

    $ifdef _DisableNACK_
    While 1 = 1
        ReturnACK = TC74_ReadStatus(TC74A4)
        HRSOut "Status: ", IBin8 ReturnACK, CR 

        $ifdef _TempOrgFormat_
        Temperature = TC74_ReadSRTemp(SensorAddress)
        $else
        Temperature = TC74_ReadSRTemp(SensorAddress, ReturnPolarity)
        $endif

        $ifdef _TempOrgFormat_
        If Temperature.7 = 1 Then          
            Temperature = ~Temperature
            Inc Temperature    
            HRSOut "Temperature: -", Dec Temperature, " Degrees C", CR
        Else                                    
            HRSOut "Temperature: +", Dec Temperature, " Degrees C", CR
        EndIf
        $else
        If ReturnPolarity = 1 Then          
            HRSOut "Temperature: -", Dec Temperature, " Degrees C", CR
        Else                                    
            HRSOut "Temperature: +", Dec Temperature, " Degrees C", CR
        EndIf        
        $endif
        HRSOut CR
        '-----------------------------------------------------------------------
        DelayMS 2000
    Wend
'-------------------------------------------------------------------------------
    $else
'-------------------------------------------------------------------------------
    While 1 = 1
        ReturnACK = TC74_ReadStatus(TC74A4)
        HRSOut "Status: ", IBin8 ReturnACK, CR

        $ifdef _TempOrgFormat_
        Temperature = TC74_ReadSRTemp(SensorAddress, ReturnACK)
        $else
        Temperature = TC74_ReadSRTemp(SensorAddress, ReturnPolarity, ReturnACK)
        $endif

        If ReturnACK = 1 Then
            HRSOut "NACK error!", CR
        Else
            HRSOut "ACK OK!", CR        
            If ReturnACK.6 = 1 Then
                HRSOut "The Sensor is Ready!", CR
            Else
                HRSOut "The Sensor is NOT Ready!", CR            
            EndIf
        EndIf

        $ifdef _TempOrgFormat_
        If Temperature.7 = 1 Then          
            Temperature = ~Temperature
            Inc Temperature    
            HRSOut "Temperature: -", Dec Temperature, " Degrees C", CR
        Else                                    
            HRSOut "Temperature: +", Dec Temperature, " Degrees C", CR
        EndIf
        $else
        If ReturnPolarity = 1 Then          
            HRSOut "Temperature: -", Dec Temperature, " Degrees C", CR
        Else                                    
            HRSOut "Temperature: +", Dec Temperature, " Degrees C", CR
        EndIf        
        $endif
        HRSOut CR
        '-----------------------------------------------------------------------
        DelayMS 2000
    Wend
    $endif
'===============================================================================
    Stop
   
    
